part of tecfy_core_client_package;

class EntityFormController extends BaseController {
  EntityService _service;
  Entity entity;
  String titleField;
  String subTitleField;
  String tailField;
  dynamic data;
  List<Property> properties;
  List<Property> calcProps;
  bool isAddNew = false;
  Map<String, List<Property>> tabs;
  bool get canSave => isAddNew && entity.permission.canAdd || !isAddNew && entity.permission.canUpdate;

  EntityFormController(
      State<StatefulWidget> state, String entityName, this.data)
      : super(state) {
    busy =false;
    _service = EntityService();
    print('-------------- EntityFormController => $entityName');
    entity = EntityService.getEntity(entityName);
    print('Entity ${entity.dbName} Can Add: ${entity.permission.canAdd} Can Edit: ${entity.permission.canUpdate}');
    properties = entity.properties.where((p) => p.isEdit).toList();
    properties.forEach((p)=>p.isVisable =true);
    calcProps = properties.where((p)=> p.propertyType == PropertyTypes.Number && (p as PropertyNumber).calcFormula != null && (p as PropertyNumber).calcFormula.isNotEmpty).toList();
    isAddNew = data == null;
    if (isAddNew) data = Map<String, dynamic>();
    initTabs();
    loadFromDb();
    
  }

  loadFromDb()async{
    if(entity.isLocal) return;
    if(data == null || data['id'] == null){
      await onValueChanged('','',true); // for default values
      return;
    }
    await Future.delayed(Duration(milliseconds: 10)); // delay a bit because you cannot show "loading" before page initlization
    setLoading();
    var result = await _service.getOne(entity.dbName,data['id'],forView: !canSave);
    setResume();
    if(result != null){
      print('------------- got(loadFromDb) -----------');
      data = result;
      await onValueChanged('','',true);
      setState();
    }

  }


  onValueChanged(value,propName,[calcAll = false])async{
    print('-----------value changed ! ($propName) $calcAll');
    try{
    if(!calcAll) data[propName] = value;

    bool updated =await entity.setDefaultValues(data);
       
    for(Property p in properties){
      var r =await p.isVisble(data,entity);
      if(r != p.isVisable){
        p.isVisable = r;
        updated=true;
      }
    }
    // check the calced properties
    for(PropertyNumber p in calcProps){
      if(p.formulaType == "Formula"){
        if(calcAll || p.calcedProps.contains(p.upperFirst(propName)) || p.calcedProps.contains(propName))
        {
          var vars = p.calcedProps.map((pro)=> data[p.lowerFirst(pro)] ?? 0).toList();
          await onValueChanged(p.getCalcResult(vars),p.dbName);
          setState();
          print('calc ${p.dbName}');
        }
      }
      else if(p.formulaType == "SumSub"){
          if(calcAll || p.sumEntityName == propName)
          {
            double val = 0;
            if(data[p.sumEntityName] != null && data[p.sumEntityName] is List){
              for(var row in data[p.sumEntityName]){
                val += row[p.sumPropertyName] ?? 0;
              }
            }
            await onValueChanged(val,p.dbName);
            setState();
          }
        }
    }

    if(updated || (!calcAll && properties.any((p)=>p.filterValueField == propName)))
      setState();
    }catch(err){
      print('XXXXXX ERROR onValueChanged: $err');
    }
  }

  bool validateEntity(){
    var result = EntityService.validateEntity(entity, data);
    if(!result.success) showMessage(result.message);
    return result.success;
  }

  Future onSave() async {

    if(!validateEntity())return;

    IEntityTransaction service = TecfyCoreApp.entityServices[entity.dbName.toLowerCase()];

    OperationResult result;
     setLoading();
    if (isAddNew){
      
      if(service != null){
        var serviceResult = await service.beforeAdd(context,data);
        if(!serviceResult.success){
          setResume();
          AppNavigator.showMessage(context, serviceResult.message ?? '',type: MessageType.error);
          return;
        }
        if(!serviceResult.skip && serviceResult.success){
          result = await EntityService.addEntity(entity,serviceResult.data);    
        }
      }
      if(result == null) result = await EntityService.addEntity(entity,data);
    }
    else
    {
      if(service != null){
        var serviceResult = await service.beforeUpdate(context,data);
        if(!serviceResult.success){
          setResume();
          AppNavigator.showMessage(context, serviceResult.message ?? '',type: MessageType.error);
          return;
        }
        if(!serviceResult.skip && serviceResult.success){
          result = await EntityService.saveEntity(entity,serviceResult.data);    
        }
      }
      if(result == null) result = await EntityService.saveEntity(entity,data);
    }
    
    setResume();

    if (!result.success) {
      AppNavigator.showMessage(
          context, result.message ?? result.errorCodeString ?? '',
          type: MessageType.error);
    } else {
      
      Navigator.pop(context,true);

      // after add or update
      if(service != null){
        OperationResultEntityTransaction serviceResult ;
        if(isAddNew)
          serviceResult  = await service.afterAdd(context,data);
        else
          serviceResult  = await service.afterUpdate(context,data);

        if(!serviceResult.success){
          AppNavigator.showMessage(context, serviceResult.message ?? '',type: MessageType.error);
          return;
        }
      }
      
      AppNavigator.showMessage(context, 'تم الحفظ بنجاح',
          type: MessageType.info);
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  void initTabs() {
    this.tabs = Map();
    for (var prop in this.entity.properties) {
      if (prop.isEdit) {
        if(prop.editTabName == null || prop.editTabName.isEmpty)
          prop.editTabName = 'التفاصيل';

        if (prop.propertyType == PropertyTypes.OtherEntityMany) {
          this.tabs[prop.name] = [prop];
        } else {
          if (this.tabs.length == 0)
            this.tabs[prop.editTabName] = [prop];
          else {
            if (this.tabs[prop.editTabName] == null) {
              this.tabs[prop.editTabName] = [prop];
            } else {
              this.tabs[prop.editTabName].add(prop);
            }
          }
        }
      }
    }
  }
}
