part of tecfy_core_client_package;

class MessageService{
  
  static bool connected = false;
  static IOWebSocketChannel channel;
  static Stream<dynamic> stream;    
  static StreamController<Message> messageStreamController ;
  static Stream<Message> messageStream ;

  static init()async{
    if(ConfigService.token == null)return;
    messageStreamController = StreamController<Message>();
    messageStream = messageStreamController.stream.asBroadcastStream();
  }

  static onMessageReceived(Message message){
    saveLocal(message);
    messageStreamController.add(message);
    // send service, confirm message recieved
    WebSocketService.sendPackage(Package(type: PackageType.confirmMessage, data: message.uid));
  }

  static sendMessage(Message message){
    WebSocketService.sendPackage(Package(type: PackageType.newMessage, data: message));
  }

  static Future<List<Message>> getLocalMessages(String userId)async{
    
    if(DatabaseService.db != null){
      var list = await DatabaseService.db.rawQuery('select * from Message where (fromUserId = $userId and toUserId = ${LocalConfigService.user.id}) or (toUserId = $userId and fromUserId = ${LocalConfigService.user.id}) order by time desc');
      return list.map((m)=>Message.fromMapDatabase(m)).toList();
    }else return [];
  }

  static Future<DateTime> getLastMessageTime()async{
    if(DatabaseService.db != null){
      var result = await DatabaseService.db.rawQuery('select time from Message order by time desc limit 1');
      if(result.length == 0) return null;
      return DateTime.fromMillisecondsSinceEpoch(result.first['time'] ?? 0);
    }else return DateTime.now();
  }

  static Future<bool> getNewMessages()async{
    try{
    var lastMessageTime = await getLastMessageTime();
    if(lastMessageTime == null) lastMessageTime = DateTime.now().add(Duration(days: -300));
    var result = await ApiService.httpGetDynamic('Message/GetLatest/${intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(lastMessageTime)}');
    if(result.success){
      List<Message> mm = result.data.map<Message>((m)=>Message.fromMapHistory(m)).toList();
      await saveLocalAll(mm);
      await markReceived(mm);
      return true;
    }else return false;
    }catch(err){
      print('XXXXXXXXXX get new messages: $err');
      return false;
    }
  }

  static Future saveLocal(Message message)async{
    try{
      if(DatabaseService.db != null)
        await DatabaseService.db.insert('Message', message.toMapDb());
    }catch(err){
      print(err);
    }
  }

  static Future saveLocalAll(List<Message> messages)async{
    if(DatabaseService.db == null) return;
    await DatabaseService.db.transaction((txn) async {
      var batch = txn.batch();
      
      messages.forEach((m)=>   
        batch.insert('Message',m.toMapDb())
      );
      
      await batch.commit(continueOnError: true);
    });

  }

  static Future<bool> markReceived(List<Message> messages)async{
    // ApiService.httpGet('')
    return true;
  }

}